/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package javax.xml.rpc;

import java.net.URL;
import java.util.Properties;
import java.security.PrivilegedAction;
import java.security.AccessController;
import javax.xml.namespace.QName;

/** The javax.xml.rpc.ServiceFactory is an abstract class that provides a
 * factory for the creation of instances of the type javax.xml.rpc.Service.
 * This abstract class follows the abstract static factory design pattern.
 * This enables a J2SE based client to create a Service instance in a portable
 * manner without using the constructor of the Service implementation class.
 * 
 * The ServiceFactory implementation class is set using the
 * javax.xml.rpc.ServiceFactory System property.
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.2 $
 */
public abstract class ServiceFactory
{
   private static ServiceFactory factory;

   public static final String SERVICEFACTORY_PROPERTY =
      "javax.xml.rpc.ServiceFactory";

   protected ServiceFactory()
   {
   }

   /** Access the ServiceFactory singleton.
    * 
    * The implementation class to be used can be overridden by setting the
    * javax.xml.rpc.ServiceFactory system property. 
    * 
    * @return The ServiceFactory singleton
    * @throws ServiceException on failure to instantiate the ServiceFactory impl
    */ 
   public static synchronized ServiceFactory newInstance()
      throws ServiceException
   {
      // Create the factory singleton if needed
      if( factory == null )
      {
         // @todo create a default ServiceFactory impl
         String defaultService = null;
         PrivilegedAction action = new PropertyAccessAction(SERVICEFACTORY_PROPERTY,
            defaultService);
         String factoryName = (String) AccessController.doPrivileged(action);
         if( factoryName == null )
         {
            throw new ServiceException("No "+SERVICEFACTORY_PROPERTY+" specified");
         }

         try
         {
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            Class factoryClass = loader.loadClass(factoryName);
            factory = (ServiceFactory) factoryClass.newInstance();
         }
         catch(Throwable e)
         {
            throw new ServiceException("Failed to create factory: "+factoryName, e);
         }
      }

      return factory;
   }

   public abstract Service loadService(Class serviceInterface)
      throws ServiceException;

   public abstract Service loadService(URL wsdlDocumentLocation,
      Class serviceInterface, Properties props) throws ServiceException;

   public abstract Service loadService(URL wsdlDocumentLocation,
      QName serviceName, Properties props) throws ServiceException;

   public abstract Service createService(QName serviceName) throws ServiceException;

   public abstract Service createService(URL wsdlDocumentLocation,
      QName serviceName) throws ServiceException;

   private static class PropertyAccessAction implements PrivilegedAction
   {
      private String name;
      private String defaultValue;
      PropertyAccessAction(String name, String defaultValue)
      {
         this.name = name;
         this.defaultValue = defaultValue;
      }
      public Object run()
      {
         return System.getProperty(name, defaultValue);
      }
   }
}
